/*
	BL test for E31 rev.4a:
	  1. BL from EPROM
	  2. continuously tests mem 0..8000H
	  3. reports results via UART-A/RS232.

*/



#define	ML_OFF			0
#define	ML_ON			1

#define	PI			3.141592

#include	<math.h>
#include	"c3x_def.h"
#include        "te3x_dsp.h"


int *USART_A	= (int *)TE3X_USART_SADDR;

int *USART_PCR	= (int *)(TE3X_USART_SADDR+0x3e);
int *USART_PVR	= (int *)(TE3X_USART_SADDR+0x3c);

#define	CCR0	0x2c
#define	CCR1	0x2d
#define	CCR2	0x2e
#define	CCR3	0x2f
#define	MODE	0x22
#define	DAFO	0x27
#define	RFC	0x28
#define	BGR	0x34
#define	STAR	0x20
#define	CMDR	0x20
#define	TIC	0x35

#define	GIS	0x38
#define	IPC	0x39
#define	ISR1	0x3b
#define	IMR1	0x3b




char welcome[]	= "E31 BL/UART#A test:  ";
char ok_mess[]	= " OK ";
char err_mess[]	= " ERROR ";



cpu_init()
 {
		// configure PBUS control register
	*C3X_PBUS_GC=TE3X_PBUS_GC_DATA;

		// set MP mode and remove on-chip ROM fm the '0' address
	TE3X_set_bmode_mp();


	*USART_PCR	= 0;  /*configure all pins as outputs */

        USART_A[CCR0]	= 0x0;
	USART_A[MODE]	= 0x0;

        USART_A[CCR0]	= 0x43; /* MCE, ASYNC */
	USART_A[CCR1]	= 0x1f; /* ODS, BCR, clk_mode=7 */
	USART_A[CCR2]	= 0x38; /* BDF, SSEL (clk_mode=7b), TCLK = out */
	USART_A[DAFO]	= 0x00; /* STOP=1, PARity=OFF, 8bit */
	USART_A[RFC]	= 0x0;  /* RFIFO threshold = 1 */
        USART_A[BGR]	= 5;	/* 9600 bauds */

        USART_A[CCR0]	|= 0x80;/* PU: set channel activity */
	USART_A[CMDR]	= 0x41; /* receiver/transmitter reset */
	USART_A[MODE]	= 0xc;	/* receiver active, set RTS */

	USART_A[IPC]	= 0x81;	/* set unmasked irqs visible, irq active low */
	USART_A[GIS]	= 0x8;	/* enable IRQS from IRQ1/USART-A */
	USART_A[IMR1]	= ~0x21;/* enable IRQS from ALLS/XPR */

 }

#define WAIT_CMDR_READY   {while (USART_A[STAR] &0x4);}

uart_transmit_char(c)
int c;
{


/* transmit using XPR irq with bits ALLS/XPR from ISR1 */
  while (!(USART_A[ISR1] &0x21));
  *USART_A=c;
  WAIT_CMDR_READY;
  USART_A[CMDR]=0x8;

}


int uart_receive_char()
{ int c;

  while (!(USART_A[STAR] &0x20));
  c=*USART_A & 0xff;
  WAIT_CMDR_READY;
  USART_A[CMDR]=0x80;	/* release RFIFO */
  return(c);
}


int uart_receiver_status()
{
  return((USART_A[STAR] & 0x20) ? 1:0);
}


uart_transmit_mess(s)
char *s;
{
  while((*s &0xff)!=0) uart_transmit_char(*(s++));
}

#define ML_OK		0
#define ML_MEM_ERR	-1



int next_cell(i)
{
  return( (((~i ^ 0x93a5 -1) <<8)  & 0xff00)
	 |(((~i ^ 0xe479 +1) <<16) & 0xff0000)
	 |(((~i ^ 0xc653 +1) <<24) & 0xff000000)
	 |((i+1) & 0xff)  );
}



int mem_test(init_value, sa, ea)
int init_value, *sa, *ea;

{ int r;
  int *ptr,i;

	r=ML_MEM_ERR;

        for(i=init_value,ptr=sa; ptr<=ea; ptr++)
	  { *ptr=i;
	    i=next_cell(i);
	  } ;
        for(i=init_value,ptr=sa; ptr<=ea; ptr++)
	  { if (*ptr !=i) return (r);
	    i=next_cell(i);
	  }
        return(ML_OK);
}





void main()
{
	int modem_status;
	int iv, *start, *end;

	start	= (int *)0;
	end	= (int *)0x07fff;

	cpu_init();

        uart_transmit_mess(welcome);


        iv = 0x3e91df48;

        while (1)
	 {


		switch(mem_test(iv,start,end))
		  {
		    case ML_OK:
		        uart_transmit_mess(ok_mess);
			break;

		    default:
		        uart_transmit_mess(err_mess);
			goto end;
			break;
		  }


	iv =    ( (((iv+1)^0x935f) <<16)
		 |(((iv-1)^0xa75a) & 0xffff)  );


	 }


end: {};

}



