/****************************************************
 USB controller test for E33.
 (C) MicroLAB Systems

 This test scans registers of USB controller, address lines,
 and generates USB interrupts.
*****************************************************/

#include "c3x_def.h"
#include "te3x_dsp.h"


init_cpu()	// DSP configure procedure
{
	*C3X_PBUS_GC=TE3X_PBUS_GC_DATA;
	C3X_CACHE_ENABLE;
}

	// USB write-to-read pause, which is required for
	// operations as WRITE to SCR register, which initializes
	// internal state machine operations and requires some
	// time to execute
	// C33 DSP running at 150MFLOPS is very fast and
	// immediate reads may return incorrect results.
wait()
{ int i;
   for (i=0;i<5;i++);
}

main()
{ int i, pass;
  unsigned d;

 for(i=0;i<100000;i++);

   init_cpu();
   TE3X_config_xf0_wdt();


   pass=0;
   while(1)
    {
	TE3X_wdt_reset();

		//-- check USB data bus via running 1<< to the SBIE reg
        for(i=0,d=1;i<=8;i++)
	 {
           TE3X_set_usb_rg(TE_USB_SBIE_RG, i==0 ? 0:d);	
           TE3X_set_usb_rg(TE_USB_SBIE1_RG, 0xff);	// perform false bus data

           if(TE3X_usb_rg(TE_USB_SBIE_RG) != (i==0 ? 0:d))
		goto error;	

           if(i>0)
	      {	d=(d<<1);
	      }
	 }

		//--- perform software reset of USB controller
		//    and verify the results of reset
           TE3X_set_usb_rg(TE_USB_SCR_RG, 0x8);	
	wait();
           TE3X_set_usb_rg(TE_USB_SCR_RG, 0);	
	wait();
           if(   TE3X_usb_rg(TE_USB_TXCNTH_RG)!=0
	      || TE3X_usb_rg(TE_USB_TXCNTL_RG)!=0
              || TE3X_usb_rg(TE_USB_SBIE_RG)!=0	
              || TE3X_usb_rg(TE_USB_SBIE1_RG)!=0	
              || TE3X_usb_rg(TE_USB_SCR_RG)!=0
             ) break;	


		//--- now toggle USB interrupt line using INT polarity bit
           TE3X_set_usb_rg(TE_USB_SCR_RG, 0x80);	
	wait();
           if(TE3X_usb_rg(TE_USB_SCR_RG)!=0x80) break;	
           TE3X_set_usb_rg(TE_USB_SCR_RG, 0x00);	
	wait();
           if(TE3X_usb_rg(TE_USB_SCR_RG)!=0x00) break;	


         	//--- now write random data to different USB registers
           TE3X_set_usb_rg(TE_USB_TXCNTH_RG, 0);	
           TE3X_set_usb_rg(TE_USB_TXCNTL_RG, 0x91);	
           TE3X_set_usb_rg(TE_USB_SBIE_RG, 0x5a);	
           TE3X_set_usb_rg(TE_USB_SBIE1_RG, 0xa5);	
           TE3X_set_usb_rg(TE_USB_FADDR_RG, 0x47);	
           TE3X_set_usb_rg(TE_USB_SCRATCH_RG, 0x6e);	
           TE3X_set_usb_rg(TE_USB_SCR_RG, 0x92);	

           if( (pass%2) ==0)
		   TE3X_set_usb_rg(TE_USB_MCSR_RG, 0x0);
	     else  TE3X_set_usb_rg(TE_USB_MCSR_RG, 0x1);


           if(TE3X_usb_rg(TE_USB_TXCNTL_RG)!=0x91) break;	
           if(TE3X_usb_rg(TE_USB_SBIE_RG)!=0x5a) break;	
           if(TE3X_usb_rg(TE_USB_SBIE1_RG)!=0xa5) break;	
           if(TE3X_usb_rg(TE_USB_FADDR_RG)!=0x47) break;	
           if(TE3X_usb_rg(TE_USB_SCRATCH_RG)!=0x6e) break;	
           if(TE3X_usb_rg(TE_USB_SCR_RG)!=0x92) break;	

           pass++;
	   if(pass>1000) pass=0;

    }


  error:
    while(1) ;

}



