/***************************************************************
  TORNADO-E3x utilities. Rev.2A
  Main 'C3x DSP environment definitions.
  (C) MicroLAB Systems Ltd, 1996-2000

  covers:
  	E31 rev.1x, rev.2A
	E33 rev.1A

  Copyright:
  ----------
	This utility is supplied free of charge as it is without
	any obligation from MicroLAB Systems. No responsibility is assumed
	for any usage or misuse of these utilities.

  http://www.mlabsys.com
  ftp://ftp.mlabsys.com
  email: techsupport@mlabsys.com
****************************************************************/

#define	ON			1
#define	OFF			0
#define	TE3X_ON			1
#define	TE3X_OFF		0

//================ E3x Memory/IO Areas ===============

	//----------- SRAM/FLASH memories ------
#define	TE3X_SRAM_SADDR		0x000000
#define	TE3X_SRAM_EADDR		0x07ffff
#define	TE3X_SRAM1_SADDR	0x100000	// SRAM alias for MC mode
#define	TE3X_SRAM1_EADDR	0x17ffff        // SRAM alias for MC mode
#define	TE3X_SRAM_LEN		0x080000

#define	TE3X_FLASH_SADDR	0x400000
#define	TE3X_FLASH_EADDR	0x4fffff
#define	TE3X_FLASH_LEN		0x100000
#define	TE3X_FLASH_1M_LEN	0x020000	// FLASH/EPROM 1M (128Kx8) length
#define	TE3X_FLASH_2M_LEN	0x040000	// FLASH/EPROM 2M (256Kx8) length	
#define	TE3X_FLASH_4M_LEN	0x080000	// FLASH/EPROM 4M (512Kx8) length
#define	TE3X_FLASH_8M_LEN	0x100000	// FLASH/EPROM 8M (1Mx8) length

	//----------- USART/USB -------------
#define TE3X_USART_SADDR	0xa00000	// r/w: SAB82532 USART (ESCC2) via D0..D7
#define TE3X_USB_SADDR		0xb00000	// E33 only: r/w: USS-825 USBC via D0..D7

	//----------- control regs ----------
#define TE3X_WDT_ENABLE_ADDR	0xd00000	// r/w: WDT-ENABLE, D0
#define TE3X_BMODE_ADDR		0x900000	// r/w: EXT_INT6 selector: D0/D1
#define TE3X_PXSX_RUN_ADDR	0xc00000	// r/w: PIOX/SIOX RESET:  D0/D1

	//----------- PIOX and SIOX rev.C ---
#define TE3X_PIOX_SADDR		0xff0000	// r/w, D0..D16 (16-bit cycles)
#define TE3X_PIOX_EADDR		0xffffff
#define TE3X_PIOX_LEN		0x010000

#define TE3X_SIOX_SADDR		0xe00000	// E33 only: SIOX rev.C r/w, D0..D7 , A0..A5
#define TE3X_SIOX_EADDR		0xe0003f
#define TE3X_SIOX_LEN		0x000040
//=====================================================

//====== recommended P-BUS/X-BUS configurations =======

#define TE3X_PBUS_GC_DATA	0x700	// external DSP READY, SRAM bank 128k
//================================================================


//********** Macros for I/O area control *************

	//--- WDT control
		//-- configure 'C3x XF0 pin for WDT reset via IOF rg
		//   !!! this is required prior first issuing the WDT reset
#define TE3X_config_xf0_wdt()		{asm(" and 60H,iof");asm(" or 2,iof");}

#define TE3X_wdt_reset()		{asm(" xor 4,iof");}
#define TE3X_set_wdt_enable(v)		(*(unsigned *) TE3X_WDT_ENABLE_ADDR)=((unsigned)v &1)
#define TE3X_wdt_enable()		((*(unsigned *) TE3X_WDT_ENABLE_ADDR) &1)


	//--- PIOX/SIOX Reset Control
#define	TE3X_PXSX_RUN_BITMASK	3
#define	TE3X_PX_RUN_BITMASK	1
#define	TE3X_PX_RESET		0
#define	TE3X_PX_RUN		1
#define	TE3X_SX_RUN_BITMASK	2
#define	TE3X_SX_RESET		0
#define	TE3X_SX_RUN		2
#define	TE3X_PX_RUN_SHIFT	0
#define	TE3X_SX_RUN_SHIFT	1

#define TE3X_set_pxsx_run(v)	(*(unsigned *)  TE3X_PXSX_RUN_ADDR)=v
#define TE3X_pxsx_run()		((*(unsigned *) TE3X_PXSX_RUN_ADDR) & TE3X_PXSX_RUN_BITMASK)

#define TE3X_set_px_run(v)	TE3X_set_pxsx_run((TE3X_pxsx_run() & ~TE3X_PX_RUN_BITMASK) | (v & TE3X_PX_RUN_BITMASK))
#define TE3X_px_run()		(TE3X_pxsx_run() & TE3X_PX_RUN_BITMASK)
#define TE3X_set_sx_run(v)	TE3X_set_pxsx_run((TE3X_pxsx_run() & ~TE3X_SX_RUN_BITMASK) | ((v & TE3X_PX_RUN_BITMASK)<<TE3X_SX_RUN_SHIFT))
#define TE3X_sx_run()		((TE3X_pxsx_run() & TE3X_SX_RUN_BITMASK)>>TE3X_SX_RUN_SHIFT)


	//--- BMODE control/status register
#define	TE3X_BMODE_START_STAT_BITMASK	1
#define	TE3X_BMODE_CURRENT_BITMASK	2
#define	TE3X_BMODE_CURRENT_SHIFT	1

#define	TE3X_BMODE_MC			1
#define	TE3X_BMODE_MP			0
#define	TE3X_BMODE_START_MC		1
#define	TE3X_BMODE_START_MP		0
#define	TE3X_BMODE_CURRENT_MC		2
#define	TE3X_BMODE_CURRENT_MP		0

#define TE3X_bmode()		((*(unsigned *) TE3X_BMODE_ADDR) & (TE3X_BMODE_START_BITMASK | TE3X_BMODE_CURRENT_BITMASK))
#define TE3X_bmode_start()	((*(unsigned *) TE3X_BMODE_ADDR) & TE3X_BMODE_START_BITMASK)
#define TE3X_bmode_current()	(((*(unsigned *) TE3X_BMODE_ADDR) & TE3X_BMODE_CURRENT_BITMASK)>>TE3X_BMODE_CURRENT_SHIFT)
#define TE3X_set_bmode_mp()	(*(unsigned *) TE3X_BMODE_ADDR)=0


	//--- SAB82532 USART (ESCC2) registers and access macros
#define TE_USART_CH_A		0	// channel-A reg base addr
#define TE_USART_CH_B		0x40	// channel-B reg base addr

#define	TE_USART_RFIFO_RG	0x0	// registers for HDLC mode
#define	TE_USART_TFIFO_RG	0x0
#define	TE_USART_STAR_RG	0x20
#define	TE_USART_CMDR_RG	0x20
#define	TE_USART_RSTA_RG	0x21
#define	TE_USART_PRE_RG		0x21
#define	TE_USART_MODE_RG	0x22
#define	TE_USART_TIMR_RG	0x23
#define	TE_USART_XAD1_RG	0x24
#define	TE_USART_XAD2_RG	0x25
#define	TE_USART_RAH1_RG	0x26
#define	TE_USART_RAH2_RG	0x27
#define	TE_USART_RAL1_RG	0x28
#define	TE_USART_RHCR_RG	0x29
#define	TE_USART_RAL2_RG	0x29
#define	TE_USART_RBCL_RG	0x2A
#define	TE_USART_XBCL_RG	0x2A
#define	TE_USART_RBCH_RG	0x2B
#define	TE_USART_XBCH_RG	0x2B
#define	TE_USART_CCR0_RG	0x2C
#define	TE_USART_CCR1_RG	0x2D
#define	TE_USART_CCR2_RG	0x2E
#define	TE_USART_CCR3_RG	0x2F
#define	TE_USART_TSAX_RG	0x30
#define	TE_USART_TSAR_RG	0x31
#define	TE_USART_XCCR_RG	0x32
#define	TE_USART_RCCR_RG	0x33
#define	TE_USART_VSTR_RG	0x34
#define	TE_USART_BGR_RG		0x34
#define	TE_USART_RLCR_RG	0x35
#define	TE_USART_AML_RG		0x36
#define	TE_USART_AMH_RG		0x37
#define	TE_USART_GIS_RG		0x38
#define	TE_USART_IVA_RG		0x38
#define	TE_USART_IPC_RG		0x39
#define	TE_USART_ISR0_RG	0x3A
#define	TE_USART_IMR0_RG	0x3A
#define	TE_USART_ISR1_RG	0x3B
#define	TE_USART_IMR1_RG	0x3B
#define	TE_USART_PVR_RG		0x3C
#define	TE_USART_PIS_RG		0x3D
#define	TE_USART_PIM_RG		0x3D
#define	TE_USART_PCR_RG		0x3E
#define	TE_USART_CCR4_RG	0x3F

#define	TE_USART_XON_RG		0x24	// ASYNC-mode register aliases
#define	TE_USART_XOFF_RG	0x25
#define	TE_USART_TCR_RG		0x26
#define	TE_USART_DAFO_RG	0x27
#define	TE_USART_RFC_RG		0x28
#define	TE_USART_TIC_RG		0x35
#define	TE_USART_MXN_RG		0x36
#define	TE_USART_MXF_RG		0x37

#define	TE_USART_SYNL_RG	0x24	// BISYNC-mode register aliases
#define	TE_USART_SYNH_RG	0x25

#define TE3X_set_usart_rg(a,v)	( *(unsigned *) (TE3X_USART_SADDR +(a&0x7f)))=((unsigned)v &0xff)
#define TE3X_usart_rg(a)	((*(unsigned *) (TE3X_USART_SADDR +(a&0x7f))) &0xff)


	//--- USS-820/825 USB registers and access macros
#define	TE_USB_TXDAT_RG		0x0
#define	TE_USB_TXCNTL_RG	0x1
#define	TE_USB_TXCNTH_RG	0x2
#define	TE_USB_TXCON_RG		0x3
#define	TE_USB_TXFLG_RG		0x4
#define	TE_USB_RXDAT_RG		0x5
#define	TE_USB_RXCNTL_RG	0x6
#define	TE_USB_RXCNTH_RG	0x7
#define	TE_USB_RXCON_RG		0x8
#define	TE_USB_RXFLG_RG		0x9
#define	TE_USB_EPINDEX_RG	0xA
#define	TE_USB_EPCON_RG		0xB
#define	TE_USB_TXSTAT_RG	0xC
#define	TE_USB_RXSTAT_RG	0xD
#define	TE_USB_SOFL_RG		0xE
#define	TE_USB_SOFH_RG		0xF
#define	TE_USB_FADDR_RG		0x10
#define	TE_USB_SCR_RG		0x11
#define	TE_USB_SSR_RG		0x12
#define	TE_USB_SBI_RG		0x14
#define	TE_USB_SBI1_RG		0x15
#define	TE_USB_SBIE_RG		0x16
#define	TE_USB_SBIE1_RG		0x17
#define	TE_USB_REV_RG		0x18
#define	TE_USB_LOCK_RG		0x19
#define	TE_USB_PEND_RG		0x1A
#define	TE_USB_SCRATCH_RG	0x1B
#define	TE_USB_MCSR_RG		0x1C

#define TE3X_set_usb_rg(a,v)	( *(unsigned *) (TE3X_USB_SADDR + (a&0x1f))) =((unsigned)v &0xff)
#define TE3X_usb_rg(a)		((*(unsigned *) (TE3X_USB_SADDR + (a&0x1f))) &0xff)


	//--- PIOX-16/SIOX_rev_C access
#define TE3X_set_px(a,v)	( *(unsigned *) (TE3X_PIOX_SADDR + (a&0xffff)))=((unsigned)v &0xffff)
#define TE3X_px(a)		((*(unsigned *) (TE3X_PIOX_SADDR + (a&0xffff))) &0xffff)

#define TE3X_set_sx(a,v)	( *(unsigned *) (TE3X_SIOX_SADDR + (a&0x3f)))=((unsigned)v &0xff)
#define TE3X_sx(a)		((*(unsigned *) (TE3X_SIOX_SADDR + (a&0x3f))) &0xff)

//********************************************************************

//*********** external API functions (file TE3X_DX.H)*****************

	//--- FLASH utilities
#define TE3X_FLASH_AMD_MANUF_ID		0x01
#define TE3X_FLASH_AM29F010_DEV_ID	0x20
#define TE3X_FLASH_AM29F040_DEV_ID	0xa4
#define TE3X_FLASH_SECTOR_PROTECTED	1
#define TE3X_FLASH_SECTOR_UNPROTECTED	0
#define TE3X_FLASH_DATA_BITMASK		0xff

int TE3X_flash_info(unsigned *manuf_id, unsigned *dev_id,
		    unsigned *len,unsigned *sectors, unsigned *sector_len);
int TE3X_flash_erase_sector(unsigned a,unsigned vf);
int TE3X_flash_erase_chip(unsigned vf);
int TE3X_flash_write(unsigned a, unsigned d);
int TE3X_flash_read(unsigned a);
int TE3X_flash_sector_wrprot(unsigned a);


	//--- general API functions
int TE3X_last_err();
int TE3X_err_flag();
int TE3X_clr_err_flag();


	//--- returned error codes
#define	TE3X_OK				0	// no errors
#define	TE3X_PARAM_ERR			-1	// erroneous function parameter
#define	TE3X_FLASH_INV_DEV_ERR		-100	// unsupported FLASH device
#define	TE3X_FLASH_ERASE_ERR		-101	// FLASH erase error
#define	TE3X_FLASH_WR_ERR		-102	// FLASH write error
#define	TE3X_FLASH_CANNOT_WR_ERR	-103	// FLASH contents does not allow to write this data
#define	TE3X_FLASH_INV_RD_ERR		-104	// invalid FLASH read data during AUTO-SELECT

//********************************************************************

